﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Globalization;

namespace MakeNK
{
    public partial class Form1 : Form
    {
        private byte[] signature = { 0x42, 0x30, 0x30, 0x30, 0x46, 0x46, 0x0a };
        private string destinationFile;

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = "vmlinux.bin";
            DialogResult result = openFileDialog1.ShowDialog();
            if (result.Equals(DialogResult.OK))
            {
                textBoxKernel.Text = openFileDialog1.FileName;
            }
        }

        private void RomFS_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = "romfs.bin";
            DialogResult result = openFileDialog1.ShowDialog();
            if (result.Equals(DialogResult.OK))
            {
                textBoxRomFS.Text = openFileDialog1.FileName;
            }
        }

        private void Zboot_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = "zboot.bin";
            DialogResult result = openFileDialog1.ShowDialog();
            if (result.Equals(DialogResult.OK))
            {
                textBoxZboot.Text = openFileDialog1.FileName;
            }
        }

        private void button1_Click_1(object sender, EventArgs e)
        {

            DialogResult result = saveFileDialog1.ShowDialog();
            if (result.Equals(DialogResult.OK))
            {
                destinationFile = saveFileDialog1.FileName;
            }
            else
            {
                MessageBox.Show("creation aborted", "abort", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            BackgroundWorker worker = new BackgroundWorker();
            worker.WorkerReportsProgress = true;
            worker.DoWork += new System.ComponentModel.DoWorkEventHandler(this.performAction);
            worker.ProgressChanged += new ProgressChangedEventHandler(updateProgress);
            worker.RunWorkerAsync();
            worker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(this.actionCompleted);
        }

        private void updateProgress(object sender, ProgressChangedEventArgs e)
        {
            //progressBar1.Value = e.ProgressPercentage;
        }

        private void actionCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Result != null)
            {
                if ((int)e.Result == 1)
                {
                    MessageBox.Show("creation completed", "completed", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
        }

        private void performAction(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;
            FileStream fpo = new FileStream(destinationFile, FileMode.Create);
            BinaryWriter w = new BinaryWriter(fpo);

            uint imagestart = 0x90010000;
            uint imageend = 0x92000000;
            uint imagelength = imageend - imagestart;
            uint nextRecordStart = imagestart + 0x40;

            uint startAddress = 0x91200000;
            uint kernelStart = 0x90020000;
            uint zbootStart = 0x91200000;
            uint romfsStart = 0x90800000;

            try
            {
                // write NK.bin header
                w.Write(signature, 0, signature.Length);
                w.Write(imagestart);
                w.Write(imagelength);

                // record 0

                MemoryStream temp = new MemoryStream();
                BinaryWriter w1 = new BinaryWriter(temp);
                w1.Write(0x43454345);
                w1.Write(imagestart + 0x50);
                w1.Close();

                byte[] record0 = temp.ToArray();

                w.Write(nextRecordStart);
                w.Write(record0.Length);
                w.Write(calcChecksum(record0));
                w.Write(record0);
                nextRecordStart += (uint)record0.Length;

                // nk.exe str  record 1
                temp = new MemoryStream();
                w1 = new BinaryWriter(temp);
                w1.Write("nk.exe".ToCharArray());
                w1.Write((ushort)0);
                w1.Close();

                byte[] record1 = temp.ToArray();
                w.Write(nextRecordStart);
                w.Write(record1.Length);
                w.Write(calcChecksum(record1));
                w.Write(record1);
                nextRecordStart += (uint)record1.Length;


                // record 2 (romhdr)

                temp = new MemoryStream();
                w1 = new BinaryWriter(temp);

                // ROMHDR start
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write(imagestart);
                w1.Write(imageend);
                w1.Write((uint)1);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((ushort)0x166);
                w1.Write((ushort)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);

                w1.Close();
                byte[] record2 = temp.ToArray();

                w.Write(nextRecordStart);
                w.Write(record2.Length);
                w.Write(calcChecksum(record2));
                w.Write(record2);
                nextRecordStart += (uint)record2.Length;



                // TOC record 3
                temp = new MemoryStream();
                w1 = new BinaryWriter(temp);

                w1.Write((uint)1);
                w1.Write((UInt64)0);
                w1.Write((uint)0);
                w1.Write(imagestart + 0x48);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Write((uint)0);
                w1.Close();
                byte[] record3 = temp.ToArray();

                w.Write(nextRecordStart);
                w.Write(record3.Length);
                w.Write(calcChecksum(record3));
                w.Write(record3);


                byte[] data = null;

                if (checkBoxZboot.Checked && !textBoxZboot.Text.Equals(""))
                {
                    FileInfo zbootFile = new FileInfo(textBoxZboot.Text);
                    if (zbootFile.Exists)
                    {
                        data = readFile(textBoxZboot.Text);
                        string adr = textBoxAdrZboot.Text.Trim();
                        if (!adr.Equals(""))
                        {
                            zbootStart = Convert.ToUInt32(adr, 16);
                        }
                        w.Write(zbootStart);
                        w.Write(data.Length);
                        w.Write(calcChecksum(data));
                        w.Write(data);
                    }
                    else
                    {
                        throw new Exception(textBoxZboot.Text + " does not exist");
                    }
                }

                if (!textBoxKernel.Text.Equals(""))
                {
                    FileInfo kernelFile = new FileInfo(textBoxKernel.Text);
                    if (kernelFile.Exists)
                    {
                        data = readFile(textBoxKernel.Text);
                        string adr = textBoxAdrKernel.Text.Trim();
                        if (!adr.Equals(""))
                        {
                            kernelStart = Convert.ToUInt32(adr, 16);
                        }
                        w.Write(kernelStart);
                        w.Write(data.Length);
                        w.Write(calcChecksum(data));
                        w.Write(data);
                    }
                    else
                    {
                        throw new Exception(textBoxKernel.Text + " does not exist");
                    }
                }

                if (checkBoxRomFs.Checked && !textBoxRomFS.Text.Equals(""))
                {
                    FileInfo romfsFile = new FileInfo(textBoxRomFS.Text);
                    if (romfsFile.Exists)
                    {
                        data = readFile(textBoxRomFS.Text);
                        string adr = textBoxAdrRomfs.Text.Trim();
                        if (!adr.Equals(""))
                        {
                            romfsStart = Convert.ToUInt32(adr, 16);
                        }
                        w.Write(romfsStart);
                        w.Write(data.Length);
                        w.Write(calcChecksum(data));
                        w.Write(data);
                    }
                    else
                    {
                        throw new Exception(textBoxRomFS.Text + " does not exist");
                    }
                }


                //for (int i = 0; i < 100; i++)
                //{
                //    worker.ReportProgress(i); 

                //    Thread.Sleep(1000);
                //}

                w.Write((uint)0);
                string adr1 = textBoxAdrStart.Text.Trim();
                if (!adr1.Equals(""))
                {
                    startAddress = Convert.ToUInt32(adr1, 16);
                }
                w.Write((uint)startAddress);
                w.Write((uint)0);

                e.Result = 1;
            }
            catch (Exception e1)
            {
                MessageBox.Show("Error: " + e1.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            w.Close();
        }

        private byte[] readFile(string name)
        {
            FileStream fs = new FileStream(name, FileMode.Open);
            long size = fs.Seek(0, SeekOrigin.End);
            byte[] data = new byte[size];
            fs.Seek(0, SeekOrigin.Begin);
            int bytesRead = fs.Read(data, 0, data.Length);
            fs.Close();
            if (bytesRead != size)
            {
                throw new Exception("file read incomplete " + name);
            }
            return data;
        }

        private int calcChecksum(byte[] array)
        {
            int result = 0;
            for (int i = 0; i < array.Length; i++)
            {
                result += array[i];
            }

            return result;
        }

        private void checkBoxRomFs_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxRomFs.Checked)
            {
                textBoxRomFS.Enabled = true;
                RomFS.Enabled = true;
                textBoxAdrRomfs.Enabled = true;
            }
            else
            {
                textBoxRomFS.Enabled = false;
                RomFS.Enabled = false;
                textBoxAdrRomfs.Enabled = false;
            }
        }

        private void checkBoxZboot_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxZboot.Checked)
            {
                textBoxZboot.Enabled = true;
                Zboot.Enabled = true;
                textBoxAdrZboot.Enabled = true;
            }
            else
            {
                textBoxZboot.Enabled = false;
                Zboot.Enabled = false;
                textBoxAdrZboot.Enabled = false;
            }
        }
    }
}
